# angular-passport
Angular module for PassportJS login and API auth. Strategies: Basic, JWT &amp; Hash

## Installation
npm install --save angular-passport

## Prerequisites
- ngCookies (<a href="https://docs.angularjs.org/api/ngCookies">https://docs.angularjs.org/api/ngCookies</a>)
- ui.router (<a href="https://github.com/angular-ui/ui-router">https://github.com/angular-ui/ui-router</a>)

<pre>
var clientApp = angular.module('clientApp', [
    'ui.router',
    'ngCookies',
    'ngPassport.basicStrategy',
    // 'ngPassport.JWTStrategy',
    // 'ngPassport.hashStrategy'
]);
</pre>

<br><br>

## Usage
### If you use Browserify
<pre>
npm install angular-passport
var ngPassportBasic = require('angular-passport').ngPassportBasic;

/* configure */
ngPassportBasic.constant('NGPASSPORT_CONF_BASIC', {
    // API_BASE_URL: 'http://192.168.1.101:9005',
    API_BASE_URL: 'http://localhost:9005',
    API_AUTH_PATHNAME: '/examples/auth/passport/basicstrategy',
    URL_AFTER_SUCCESSFUL_LOGIN: '/examples-spa/login/basic/page1',
    URL_AFTER_LOGOUT: '/examples-spa/login/basic/form'
});
</pre>

### If you include it into HTML file (compiled version)
<pre>
&lt;script src=".../dist/js/ngPassport.js"&gt;&lt;/script&gt;
&lt;script&gt;
    ngPassportHash.constant('NGPASSPORT_CONF_HASH', {
        API_BASE_URL: 'http://localhost:9005',
        API_AUTH_PATHNAME: '/examples/auth/passport/hashstrategy-gethash',
        URL_AFTER_SUCCESSFUL_LOGIN: '/examples-spa/login/hash/page1',
        URL_AFTER_LOGOUT: '/examples-spa/login/hash/form'
    });
&lt;/script&gt;
</pre>


<br><br>
<hr>


<section class="main-block">

	<div class="row">
		<div class="col-sm-12">
			<h1>Examples</h1>

			<!-- BASIC -->
			<h3><u>Basic Authentication</u></h3>
				1. When user type username &amp; password in login form a string 'username:password' is encoded to base64 (john:test  =>  am9objp0ZXN0)
			<br>2. A request with header: <b>Authorization: Basic am9objp0ZXN0</b> is sent to API.
			<br>3. If auth was successful API returns object
<pre>
{
 "isLoggedIn": true,
 "msg": "Basic authentication was succcessfull.",
   "putLocally": {
   		"username": "john",
   		"authHeader": "Basic am9objp0ZXN0"
  	}
}
</pre>
  		    <br>4. A cookie 'authAPI' is set with object 'putLocally'
  		    <br>5. Angular sets up interceptor with cookie data and each $http request sent to API is intercepted with 'Authorization':'Basic am9objp0ZXN0' header

			<br><br>
			<a ui-sref="examples-spa_login_basic_form">Try Basic Auth example ...</a>


			<!-- JWT -->
			<h3 style="margin-top:80px"><u>JWT Authentication</u></h3>
				1. User type username &amp; password and when Login button is clicked request
					<b>POST /examples/auth/passport/jwtstrategy-gettoken {username: 'john', password: 'test'}</b> is sent to API
			<br>2. Client receives respond object from API
<pre>
{
  "isLoggedIn": true,
  "msg": "Login was successful. JWT Token is generated and you can use it in request header to access API. Authorization: JWT eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjU3YTcyNjk1MzcwYmM1MDk2MmUzNDVmZSIsImlhdCI6MTQ3Mzg2MDgyM30.R-OPMVRUXgZ2RK4iPkmEMWGbKg7iN_zJj1MA1cGBHWY",
  "putLocally": {
    "username": "john",
    "authHeader": "JWT eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjU3YTcyNjk1MzcwYmM1MDk2MmUzNDVmZSIsImlhdCI6MTQ3Mzg2MDgyM30.R-OPMVRUXgZ2RK4iPkmEMWGbKg7iN_zJj1MA1cGBHWY"
  }
}
</pre>
			<br>3. Object putLocally is used to create cookie 'authAPI'
			<br>4. Angular sets up interceptor with cookie data and each $http request sent to API is intercepted with 'Authorization': 'JWT eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjU3YTcyNjk1MzcwYmM1MDk2MmUzNDVmZSIsImlhdCI6MTQ3Mzg2MDgyM30.R-OPMVRUXgZ2RK4iPkmEMWGbKg7iN_zJj1MA1cGBHWY'

			<br><br>
			<a ui-sref="examples-spa_login_jwt_form">Try JWT Auth example ...</a>


			<!-- HASH -->
			<h3 style="margin-top:80px"><u>Hash Authentication</u></h3>
			Notice: Hash string must be stored in database and returned by API on user's successful login
			<br>1. User type username &amp; password and when Login button is clicked request
					<b>POST /examples/auth/passport/hashstrategy-gethash {username: 'john', password: 'test'}</b> is sent to API
			<br>2. Client receives respond object from API
<pre>
{
  "isLoggedIn": true,
  "msg": "Login was successful. Now you can use hash to access API endpoints. For example: /examples/auth/passport/hashstrategy/e7b1951a91718085f4382391c31ef175df72addddb",
  "putLocally": {
    "username": "john",
    "hash": "e7b1951a91718085f4382391c31ef175df72addddb"
  }
}
</pre>
			<br>3. Object putLocally is used to create cookie 'authAPI'
			<br>4. Angular sets up interceptor with cookie data and each $http request sent to API is intercepted by adding suffix to URL
				<br>for example: <b>GET /examples/auth/passport/hashstrategy/e7b1951a91718085f4382391c31ef175df72addddb</b>


			<br><br>
			<a ui-sref="examples-spa_login_hash_form">Try Hash Auth example ...</a>

		</div>
	</div>





	<div class="row" style="margin-top:50px;">
		<div class="col-sm-12">
			<h3>Preconditions</h3>
			1. Directive will send 'username' and 'password' object properties to API: {username: 'john', password: 'test'} .
			It is also possible to define some other variable names for example 'user', 'pass' if your API requires so.
			<br>2. API response object when username and password are correct <u>must have this format</u>:
			{	"isLoggedIn": true,
  				"msg": "Basic authentication was succcessfull.",
  				"putLocally": {
    				"username": "john",
    				"authHeader": "Basic am9objp0ZXN0"
  			}
  			<br>3. Cookie will be set with 'putLocally' object, so it must have 'username' property.
  			<br>4. configuration
				ngPassportBasic.constant('NGPASSPORT_CONF_BASIC', {
				    API_BASE_URL: 'http://localhost:9005',
				    API_AUTH_PATHNAME: '/examples/auth/passport/basicstrategy',
				    URL_AFTER_SUCCESSFUL_LOGIN: '/examples-spa/login/page1',
				    URL_AFTER_LOGOUT: '/examples-spa/login/form'
				});

		</div>
	</div>




</section>
